type id = string

type operator = Add | Sub | Mul | Div | Not | And | Eq | Gt | If

type t =
  | Int    of int
  | Bool   of bool
  | Var    of id
  | Op     of operator * (t list)
  | Let    of id * t * t
  | LetRec of id * t * t
  | Fun    of id * t
  | App    of t * t



let rec print e s =
  print_string s;
  match e with
    | Int x ->
	print_string "Int: "; print_int x; print_newline ()
    | Bool b ->
	print_string "Bool: ";
	print_string (if b then "true" else "false"); print_newline ()
    | Var id ->
	print_string ("Var: "^id); print_newline ()
    | Op (op,l) ->
	print_string ("Op: "^(match op with
				| Add -> "Add"
				| Sub -> "Sub"
				| Mul -> "Mul"
				| Div -> "Div"
				| Not -> "Not"
				| And -> "And"
				| Eq  -> "Eq"
				| Gt  -> "Gt"
				| If  -> "If"));
	print_newline ();
	List.iter (fun e -> print e (s^"  ")) l
    | Let (id,e,e') ->
	print_string ("Let: "^id); print_newline ();
	print e (s^"  ");
	print e' (s^"  ")
    | LetRec (id,e,e') ->
	print_string ("LetRec: "^id); print_newline ();
	print e (s^"  ");
	print e' (s^"  ")
    | Fun (id,e) ->
	print_string ("Fun: "^id); print_newline ();
	print e (s^"  ")
    | App (e,e') ->
	print_string "App:"; print_newline ();
	print e (s^"  ");
	print e' (s^"  ")
